<?php

namespace AppBundle\Controller\Admin;

use AppBundle\Entity\Property;
use AppBundle\Entity\PropertyImage;
use AppBundle\Entity\PropertyMeta;
use AppBundle\Form\PropertyType;
use AppBundle\Helpers\ImagesHelper;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\Form\Form;
use Symfony\Component\HttpFoundation\Request;

/**
 * Admin\Property controller.
 *
 * @Route("/admin")
 */
class PropertyController extends BaseController
{

    /**
     * Lists all Business\Property entities.
     *
     * @Route("/", name="admin_property_index")
     * @Method("GET")
     * @Template()
     */
    public function indexAction()
    {        
        $entities = $this->getProperties()->findAll();

        return array(
            'entities' => $entities,
        );
    }
    /**
     * Creates a new Business\Property entity.
     *
     * @Route("/property/", name="admin_property_create")
     * @Method("POST")
     * @Template("AppBundle:Admin\Property:new.html.twig")
     */
    public function createAction(Request $request)
    {
        $entity = new Property();
        $form = $this->createCreateForm($entity);
        $form->handleRequest($request);

        if ($form->isValid()) {
            $this->SaveEntity($entity);
            
            $id = $entity->getId();
            
            $this->saveMeta($request, $entity);
            $this->saveImages($request, $entity);

            return $this->redirect($this->generateUrl('admin_property_edit', array('id' => $id)));
        }

        return array(
            'entity' => $entity,
            'form'   => $form->createView(),
        );
    }

    /**
     * Creates a form to create a Business\Property entity.
     *
     * @param Property $entity The entity
     *
     * @return Form The form
     */
    private function createCreateForm(Property $entity)
    {
        $form = $this->createForm(new PropertyType(), $entity, array(
            'action' => $this->generateUrl('admin_property_create'),
            'method' => 'POST',
        ));

        return $form;
    }

    /**
     * Displays a form to create a new Business\Property entity.
     *
     * @Route("/property/new", name="admin_property_new")
     * @Method("GET")
     * @Template()
     */
    public function newAction()
    {
        $entity = new Property();
        $form   = $this->createCreateForm($entity);
        $options = $this->getPropertyMetaOptions()->findAllOrdered();

        return array(
            'entity' => $entity,
            'form'   => $form->createView(),
            'options' => $options,
        );
    }

    /**
     * Displays a form to edit an existing Business\Property entity.
     *
     * @Route("/property/{id}/edit", name="admin_property_edit")
     * @Method("GET")
     * @Template()
     */
    public function editAction($id)
    {
        $entity = $this->getProperties()->findForEdit($id);

        if (!$entity) {
            throw $this->createNotFoundException('Unable to find Business\Property entity.');
        }

        $editForm = $this->createEditForm($entity);
        $deleteForm = $this->createDeleteForm($id);
        
        $options = $this->getPropertyMetaOptions()->findAllOrdered();

        return array(
            'entity'      => $entity,
            'edit_form'   => $editForm->createView(),
            'delete_form' => $deleteForm->createView(),
            'options' => $options,
            'id' => $id,
        );
    }

    /**
    * Creates a form to edit a Business\Property entity.
    *
    * @param Property $entity The entity
    *
    * @return Form The form
    */
    private function createEditForm(Property $entity)
    {
        $form = $this->createForm(new PropertyType(), $entity, array(
            'action' => $this->generateUrl('admin_property_update', array('id' => $entity->getId())),
            'method' => 'PUT',
        ));

        return $form;
    }
    /**
     * Edits an existing Business\Property entity.
     *
     * @Route("/property/{id}", name="admin_property_update")
     * @Method("PUT")
     * @Template("AppBundle:Admin\Property:edit.html.twig")
     */
    public function updateAction(Request $request, $id)
    {                
        $entity = $this->getProperties()->find($id);

        if (!$entity) {
            throw $this->createNotFoundException('Unable to find Business\Property entity.');
        }

        $deleteForm = $this->createDeleteForm($id);
        $editForm = $this->createEditForm($entity);
        $editForm->handleRequest($request);

        if ($editForm->isValid()) {
            $this->UpdateEntity();
            
            $this->saveMeta($request, $entity);
            $this->saveImages($request, $entity);
            
            return $this->redirect($this->generateUrl('admin_property_edit', array('id' => $id)));
        }

        return array(
            'entity'      => $entity,
            'edit_form'   => $editForm->createView(),
            'delete_form' => $deleteForm->createView(),
        );
    }
    /**
     * Deletes a Business\Property entity.
     *
     * @Route("/property/{id}", name="admin_property_delete")
     * @Method("DELETE")
     */
    public function deleteAction(Request $request, $id)
    {
        $form = $this->createDeleteForm($id);
        $form->handleRequest($request);

        if ($form->isValid()) {
            $entity = $this->getProperties()->find($id);

            if (!$entity) {
                throw $this->createNotFoundException('Unable to find Business\Property entity.');
            }
            
            foreach ($entity->getMeta() as $m) {
                $this->RemoveEntity($m);
            }
            
            foreach ($entity->getContacts() as $c) {
                $this->RemoveEntity($c);
            }
            
            foreach ($entity->getTags() as $t) {
                $this->RemoveEntity($t);
            }
            
            foreach ($entity->getImages() as $i) {
                $this->RemoveEntity($i);
                
                // * Borrar imagenes
            }
            
            $this->RemoveEntity($entity);
        }

        return $this->redirect($this->generateUrl('admin_property_index'));
    }

    /**
     * Creates a form to delete a Business\Property entity by id.
     *
     * @param mixed $id The entity id
     *
     * @return Form The form
     */
    private function createDeleteForm($id)
    {
        return $this->createFormBuilder()
            ->setAction($this->generateUrl('admin_property_delete', array('id' => $id)))
            ->setMethod('DELETE')
            ->getForm()
        ;
    }
    
    
    private function saveMeta(Request $request, $entity) {
        
        foreach ($entity->getMeta() as $m) {
            $this->RemoveEntity($m);
        }
        
        if (!$request->request->has('meta'))
            return null;
        
        foreach ($request->request->get('meta') as $key => $value) {            
            
            if ($value != null){
                
                $option = $this->getPropertyMetaOptions()->findOneByKey($key);
                
                $pm = new PropertyMeta();
                $pm->setKey($key);
                $pm->setOption($option);
                $pm->setValue($value);
                $pm->setProperty($entity);
                $this->SaveEntity($pm);
                
            }
            
        }
        
    }
    
    private function saveImages(Request $request, $entity) {
        
        if (!$request->request->has('img'))
            return null;
        
        $i = 0;
        
        foreach ($request->request->get('img') as $key => $value) {            
            
            switch ($value) {
                case "add":
                    $i++;
                    
                    $filename = ImagesHelper::GenerateImages($key);
                    
                    if ($filename != null) {
                        $pi = new PropertyImage();
                        $pi->setFile($key);
                        $pi->setOrder($i);
                        $pi->setProperty($entity);
                        $this->SaveEntity($pi);
                    }
                    break;
                
                case "remove":
                    foreach ($entity->getImages() as $pi) {
                        if ($pi->getFile() == $key) {                            
                            if (ImagesHelper::DeleteImages($key)) {
                                $this->RemoveEntity($pi);
                            }                            
                        }                            
                    }
                    break;
                
                default:
                    $i++;
                    foreach ($entity->getImages() as $pi) {
                        if ($pi->getFile() == $key && $pi->getOrder() != $i) {
                            $pi->setOrder($i);
                            $this->UpdateEntity();
                        }
                    }
                    break;
            }
                        
        }        
    }   
   
}
