<?php

namespace AppBundle\Controller\Web;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;

class PropertyController extends Controller
{
    const pageSize = 8;
    
    protected function getProperties() {              
        return $this->getDoctrine()->getRepository('AppBundle:Property');
    }
    
    protected function getPropertyMetas() {              
        return $this->getDoctrine()->getRepository('AppBundle:PropertyMeta');
    }
    
    /**
     * @Route("/propiedades", name="web_property_index")
     * @Template
     */
    public function indexAction()
    {
        $properties = $this->getProperties()->findPublishedWithOffset(0, $this::pageSize);
        
        $totalCount = $this->getProperties()->getPublishedCount();
        
        return array('properties' => $properties, 'total' => $totalCount, 'pageSize' => $this::pageSize);
    }    
    
    /**
     * @Route("/propiedades/cargar/{page}", name="web_property_load", defaults = { "page" = "1" })
     * @Template
     */
    public function loadAction($page)
    {
        $properties = $this->getProperties()->findPublishedWithOffset(($page - 1) * $this::pageSize, $this::pageSize);
                
        return array('properties' => $properties, 'page' => $page);
    }    
    
    /**
     * @Route("/ver/{id}/{slug}", name="web_property_view", defaults = { "slug" = "" })
     * @Template
     */
    public function viewAction($id, $slug = "")
    {
        $property = $this->getProperties()->findForView($id);
                
        return array('property' => $property);
    }
    
    /**
     * @Template("@App/Web/Default/featured.html.twig")
     */
    public function featuredAction()
    {
        $properties = $this->getProperties()->findForFeatured();
        
        $categories = $this->getPropertyMetas()->findAllValuesWithPropertiesForKey('Tipo de propiedad');
        
        return array('featured' => $properties, 'categories' => $categories);
    }
    
    /**
     * @Template("@App/Web/Default/recent.html.twig")
     */
    public function recentAction()
    {
        $properties = $this->getProperties()->findPublishedWithOffset(0, 8);
        
        return array('recent' => $properties);
    }
    
    /**
     * @Template("@App/Web/Default/popular.html.twig")
     */
    public function popularAction()
    {
        $properties = $this->getProperties()->findPublishedWithOffset(0, 3);
        
        return array('popular' => $properties);
    }
    
    /**
     * @Template("@App/Web/Property/related.html.twig")
     */
    public function relatedAction()
    {
        $properties = $this->getProperties()->findForFeatured(3);
        
        return array('related' => $properties);
    }
}
