<?php

namespace AppBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Criteria;

use AppBundle\Helpers\UrlHelper;

/**
 * @ORM\Entity(repositoryClass="AppBundle\Repository\PropertyRepository")
 * @ORM\Table(name="app_property")
 */
class Property {

    /**
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $title;

    /**
     * @ORM\Column(type="string", length=500, name="brief_description", nullable=true)
     */
    protected $briefDescription;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $description;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $address;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $geolocation;
    
    /**
     * @ORM\Column(type="boolean")
     */
    protected $published;

    /**
     * @ORM\Column(type="boolean")
     */
    protected $featured;

    /**
     * @ORM\ManyToOne(targetEntity="AppBundle\Entity\City", inversedBy="properties")
     * @ORM\JoinColumn(name="city_id", referencedColumnName="id")
     */
    protected $city;

    /**
     * @ORM\OneToMany(targetEntity="AppBundle\Entity\PropertyMeta", mappedBy="property")
     */
    protected $meta;

    /**
     * @ORM\OneToMany(targetEntity="AppBundle\Entity\PropertyImage", mappedBy="property")
     */
    protected $images;

    /**
     * @ORM\OneToMany(targetEntity="AppBundle\Entity\Contact", mappedBy="property")
     */
    protected $contacts;
    
    /**
     * @ORM\ManyToMany(targetEntity="AppBundle\Entity\Tag", inversedBy="properties")
     * @ORM\JoinTable(name="app_property_tag")
     */
    protected $tags;

    public function __construct() {
        $this->meta = new ArrayCollection();
        $this->tags = new ArrayCollection();
        $this->images = new ArrayCollection();
        $this->contacts = new ArrayCollection();
    }

    // ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ Codigo autogenerado ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓

    /**
     * Get id
     *
     * @return integer 
     */
    public function getId() {
        return $this->id;
    }

    /**
     * Set title
     *
     * @param string $title
     * @return Property
     */
    public function setTitle($title) {
        $this->title = $title;

        return $this;
    }

    /**
     * Get title
     *
     * @return string 
     */
    public function getTitle() {
        return $this->title;
    }

    /**
     * Set briefDescription
     *
     * @param string $briefDescription
     * @return Property
     */
    public function setBriefDescription($briefDescription) {
        $this->briefDescription = $briefDescription;

        return $this;
    }

    /**
     * Get briefDescription
     *
     * @return string 
     */
    public function getBriefDescription() {
        return $this->briefDescription;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return Property
     */
    public function setDescription($description) {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string 
     */
    public function getDescription() {
        return $this->description;
    }

    /**
     * Set address
     *
     * @param string $address
     * @return Property
     */
    public function setAddress($address) {
        $this->address = $address;

        return $this;
    }

    /**
     * Get address
     *
     * @return string 
     */
    public function getAddress() {
        return $this->address;
    }
    
    /**
     * Set geolocation
     *
     * @param string $geolocation
     * @return Property
     */
    public function setGeolocation($geolocation) {
        $this->geolocation = $geolocation;

        return $this;
    }

    /**
     * Get geolocation
     *
     * @return string 
     */
    public function getGeolocation() {
        return $this->geolocation;
    }

    /**
     * Set published
     *
     * @param boolean $published
     * @return Property
     */
    public function setPublished($published) {
        $this->published = $published;

        return $this;
    }

    /**
     * Get published
     *
     * @return boolean 
     */
    public function getPublished() {
        return $this->published;
    }

    /**
     * Set featured
     *
     * @param boolean $featured
     * @return Property
     */
    public function setFeatured($featured) {
        $this->featured = $featured;

        return $this;
    }

    /**
     * Get featured
     *
     * @return boolean 
     */
    public function getFeatured() {
        return $this->featured;
    }

    /**
     * Set city
     *
     * @param \AppBundle\Entity\City $city
     * @return Property
     */
    public function setCity(\AppBundle\Entity\City $city = null) {
        $this->city = $city;

        return $this;
    }

    /**
     * Get city
     *
     * @return \AppBundle\Entity\City 
     */
    public function getCity() {
        return $this->city;
    }

    /**
     * Add meta
     *
     * @param \AppBundle\Entity\PropertyMeta $meta
     * @return Property
     */
    public function addMetum(\AppBundle\Entity\PropertyMeta $meta) {
        $this->meta[] = $meta;

        return $this;
    }

    /**
     * Remove meta
     *
     * @param \AppBundle\Entity\PropertyMeta $meta
     */
    public function removeMetum(\AppBundle\Entity\PropertyMeta $meta) {
        $this->meta->removeElement($meta);
    }

    /**
     * Get meta
     *
     * @return \Doctrine\Common\Collections\Collection 
     */
    public function getMeta() {
        return $this->meta;
    }

    /**
     * Add images
     *
     * @param \AppBundle\Entity\PropertyImage $images
     * @return Property
     */
    public function addImage(\AppBundle\Entity\PropertyImage $images) {
        $this->images[] = $images;

        return $this;
    }

    /**
     * Remove images
     *
     * @param \AppBundle\Entity\PropertyImage $images
     */
    public function removeImage(\AppBundle\Entity\PropertyImage $images) {
        $this->images->removeElement($images);
    }

    /**
     * Get images
     *
     * @return \Doctrine\Common\Collections\Collection 
     */
    public function getImages() {
        return $this->images;
    }

    /**
     * Add contacts
     *
     * @param \AppBundle\Entity\Contact $contacts
     * @return Property
     */
    public function addContact(\AppBundle\Entity\Contact $contacts) {
        $this->contacts[] = $contacts;

        return $this;
    }

    /**
     * Remove contacts
     *
     * @param \AppBundle\Entity\Contact $contacts
     */
    public function removeContact(\AppBundle\Entity\Contact $contacts) {
        $this->contacts->removeElement($contacts);
    }

    /**
     * Get contacts
     *
     * @return \Doctrine\Common\Collections\Collection 
     */
    public function getContacts() {
        return $this->contacts;
    }
    
    /**
     * Add tags
     *
     * @param \AppBundle\Entity\Tag $tags
     * @return Property
     */
    public function addTag(\AppBundle\Entity\Tag $tags) {
        $this->tags[] = $tags;

        return $this;
    }

    /**
     * Remove tags
     *
     * @param \AppBundle\Entity\Tag $tags
     */
    public function removeTag(\AppBundle\Entity\Tag $tags) {
        $this->tags->removeElement($tags);
    }

    /**
     * Get tags
     *
     * @return \Doctrine\Common\Collections\Collection 
     */
    public function getTags() {
        return $this->tags;
    }

    // ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ Codigo extra ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓

    /**
     * Get title for url
     *
     * @return string
     */
    public function getSlug() {
        return UrlHelper::cleanForShortURL($this->title);
    }    
    
    /**
     * Get featured image
     *
     * @return \AppBundle\Entity\PropertyImage 
     */
    public function getFeaturedImage() {
        $criteria = Criteria::create()->orderBy(array("order" => Criteria::ASC));

        if ($this->images->matching($criteria)->isEmpty()) {
            return array('file' => 'ph.png');
        }

        return $this->images->matching($criteria)->first();
    }

    /**
     * Get meta value
     *
     * @return boolean
     */
    public function getHas($key) {
        $criteria = Criteria::create()->where(Criteria::expr()->eq('key', $key));

        return $this->meta->matching($criteria)->count() == 1;
    }
    
    /**
     * Get meta value
     *
     * @return boolean
     */
    public function getContains($key, $value) {
        $criteria = Criteria::create()->where(Criteria::expr()->eq('key', $key))->andWhere(Criteria::expr()->contains('value', $value));

        return $this->meta->matching($criteria)->count() == 1;
    }

    /**
     * Get meta value
     *
     * @return string
     */
    public function get_($key) {
        $criteria = Criteria::create()->where(Criteria::expr()->eq('key', $key));

        return $this->meta->matching($criteria)->first();
    }

    /**
     * Get tags
     *
     * @return array
     */
    public function getAllTags() {
        if ($this->tags == null)
            return null;

        $a = array();

        foreach ($this->tags as $t) {
            $a[] = $t->getId();
        }

        return $a;
    }

    /**
     * Get meta value descriptions
     *
     * @return array
     */
    public function getAllMetaDescriptions() {
        if ($this->meta == null)
            return null;
        
        $a = array();
        
        foreach ($this->meta as $pm) {
            switch ($pm->getOption()->getType()) {
                case 'choice':
                    $a[] = str_replace(' ', '', $pm->getValue());
                    break;
                case 'multichoice':
                    foreach ($pm->getChoices() as $c) {
                        $a[] = str_replace(' ', '', $c);
                    }
                    break;
                case 'boolean':
                    $a[] = str_replace(' ', '', $pm->getKey());
                    break;
            }
        }

        return $a;
    }
    
    /**
     * Get meta value descriptions
     *
     * @return array
     */
    public function getAllMetaNumeric() {
        if ($this->meta == null)
            return null;
        
        $a = array();
        
        foreach ($this->meta as $pm) {
            switch ($pm->getOption()->getType()) {
                case 'number':
                    $a[] = $pm->getKey() . ':' . filter_var($pm->getValue(), FILTER_SANITIZE_NUMBER_INT);
                    break;
                case 'special':
                    $sp = $pm->getOption()->getSpecial();
                    if($sp['type'] == 'number') {
                        $a[] = $pm->getKey() . ':' . filter_var($pm->getValue(), FILTER_SANITIZE_NUMBER_INT);
                    }
                    break;
            }
        }

        return $a;
    }
    
    /**
     * Get address for map
     *
     * @return string
     */
    public function getMapAddress() {
        return urlencode($this->address . " " . $this->city->getName());
    }
}
