<?php

namespace AppBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;

/**
 * @ORM\Entity(repositoryClass="AppBundle\Repository\PropertyMetaOptionRepository")
 * @ORM\Table(name="app_property_meta_option")
 */
class PropertyMetaOption {

    /**
     * @ORM\Id
     * @ORM\Column(type="string", length=255, name="`key`")
     */
    protected $key;

    /**
     * @ORM\Column(type="text", name="`option`")
     */
    protected $option;
    
    /**
     * @ORM\Column(type="integer")
     */
    protected $order;

    /**
     * @ORM\OneToMany(targetEntity="AppBundle\Entity\PropertyMeta", mappedBy="option")
     */
    protected $meta;

    public function __construct() {
        $this->meta = new ArrayCollection();
    }

    // ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ Codigo autogenerado ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    
    /**
     * Set key
     *
     * @param string $key
     * @return DynamicVariable
     */
    public function setKey($key)
    {
        $this->key = $key;

        return $this;
    }

    /**
     * Get key
     *
     * @return string 
     */
    public function getKey()
    {
        return $this->key;
    }

    /**
     * Set option
     *
     * @param string $option
     * @return DynamicVariable
     */
    public function setOption($option)
    {
        $this->option = $option;

        return $this;
    }

    /**
     * Get option
     *
     * @return string 
     */
    public function getOption()
    {
        return $this->option;
    }       

    /**
     * Set order
     *
     * @param integer $order
     * @return PropertyMetaOption
     */
    public function setOrder($order) {
        $this->order = $order;

        return $this;
    }

    /**
     * Get order
     *
     * @return integer 
     */
    public function getOrder() {
        return $this->order;
    }

    // ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ Codigo extra ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓

    /**
     * Get type
     *
     * @return string 
     */
    public function getType() {
        /*
         * Los tipos son:
         * * text
         * * number
         * * boolean
         * * choice
         */

        $o = $this->option;

        if ($o == 'text') {

            return 'text';
        } elseif ($o == 'number') {

            return 'number';
        } elseif ($o == 'boolean') {

            return 'boolean';
        } elseif (strpos($o, '[') === 0) {

            return 'choice';
        } elseif (strpos($o, '*[') === 0) {

            return 'multichoice';
        } elseif (strpos($o, 'special') === 0) {

            return 'special';
        }

        return 'text';
    }

    /**
     * Get choices
     *
     * @return array 
     */
    public function getChoices() {
        if ($this->getType() != 'choice' && $this->getType() != 'multichoice') {
            return null;
        }

        return explode('|', trim($this->option, '*[]'));
    }

    /**
     * Get choices
     *
     * @return array 
     */
    public function getUsedChoices() {
        $c = $this->getChoices();
        if ($c == null) {
            return null;
        }

        $uc = array();

        foreach ($this->meta as $m) {
            foreach ($c as $choice) {
                if ($m->getChoices() != null) {
                    if (in_array($choice, $m->getChoices())) {
                        if (!in_array($choice, $uc)) {
                            $uc[] = $choice;
                        }
                    }
                } else if ($choice == $m->getValue()) {
                    if (!in_array($choice, $uc)) {
                        $uc[] = $choice;
                    }
                }
            }
        }

        return $uc;
    }

    /**
     * Get choices
     *
     * @return array 
     */
    public function getSpecial() {
        if ($this->getType() != 'special') {
            return null;
        }

        $value = explode(':', $this->option);

        return array('type' => $value[1], 'prefix' => $value[2]);
    }

}
