<?php

namespace AppBundle\Repository;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Tools\Pagination\Paginator;

class PropertyRepository extends EntityRepository
{
    public function findAllPublished()
    {
        return $this->createQueryBuilder('p')
                    ->select('p, pi')
                    ->leftJoin('p.images', 'pi')
                    ->andWhere('p.published = 1')
                    ->getQuery()
                    ->getResult();
    }
    
    public function findPublishedWithOffset($offset = 0, $limit = 5)
    {
        $query = $this->createQueryBuilder('p')
                    ->select('p, pi, t, pm')
                    ->leftJoin('p.images', 'pi')                    
                    ->leftJoin('p.tags', 't')
                    ->leftJoin('p.meta', 'pm')
                    ->leftJoin('pm.option', 'pmo')
                    ->where('p.published = 1')
                    ->orderBy('p.id', 'DESC')      
                    ->addOrderBy('pi.order', 'ASC')
                    ->setMaxResults($limit)
                    ->setFirstResult($offset)
                    ->getQuery();
        
        return new Paginator($query);
    }
    
    public function findAllForIndex()
    {
        return $this->createQueryBuilder('p')
                    ->select('p, pi, pm, t')
                    ->leftJoin('p.images', 'pi')
                    ->leftJoin('p.meta', 'pm')
                    ->leftJoin('p.tags', 't')
                    ->andWhere('p.published = 1')
                    ->getQuery()
                    ->getResult();
    }
    
    public function findForFeatured($limit = 9)
    {
        $query =  $this->createQueryBuilder('p')
                    ->select('p, pi, pm, pmo')
                    ->leftJoin('p.images', 'pi')
                    ->leftJoin('p.meta', 'pm')
                    ->leftJoin('pm.option', 'pmo')
                    ->andWhere('p.featured = 1')
                    ->andWhere('p.published = 1')
                    ->orderBy('p.id', 'DESC')
                    ->addOrderBy('pi.order', 'ASC')
                    ->setMaxResults($limit)
                    ->setFirstResult(0)
                    ->getQuery();
        
        return new Paginator($query);
    }
    
    public function findAllByQuery($filter)
    {
        $keywords = preg_split("/[\s,-]+/", $filter);
        
        $query = $this->createQueryBuilder('p')
                    ->select('p, pi, pm, t')
                    ->leftJoin('p.images', 'pi')
                    ->leftJoin('p.meta', 'pm')
                    ->leftJoin('p.tags', 't')
                    ->where('p.id = 0');

        $i = 0;
        
        foreach ($keywords as $kw) {
            $query = $query->orWhere('p.title LIKE :filter' . $i)
                    ->orWhere('p.description LIKE :filter' . $i)
                    ->orWhere('p.briefDescription LIKE :filter' . $i)
                    ->orWhere('p.address LIKE :filter' . $i)
                    ->orWhere('pm.value LIKE :filter' . $i)
                    ->orWhere('pm.key LIKE :filter' . $i)
                    ->orWhere('t.value LIKE :filter' . $i)
                    ->setParameter('filter' . $i, '%' . $kw . '%');
            $i++;
        }
        
        return $query->andWhere('p.published = 1')->getQuery()->getResult();     
    }
    
    public function findAllByFilter(\AppBundle\Filter\PropertyFilter $filter)
    {
        $query = $this->createQueryBuilder('p')
                    ->select('p, pi, t, c')
                    ->leftJoin('p.images', 'pi')
                    ->leftJoin('p.meta', 'pm')
                    ->leftJoin('p.tags', 't')
                    ->leftJoin('p.city', 'c')
                    ->where('p.id > 0');
        
        if ($filter != null) {
            $query = $query->andWhere("c.id = :city OR :city IS NULL")
                    ->andWhere("(pm.key = 'Tipo de operación' AND pm.value = :operationType) OR :operationType IS NULL")
                    ->andWhere("(pm.key = 'Tipo de propiedad' AND pm.value = :propertyType) OR :propertyType IS NULL")
                    ->andWhere("(pm.key = 'Ambientes' AND pm.value = :bedrooms) OR :bedrooms IS NULL")
                    ->andWhere("(pm.key = 'Baños' AND pm.value = :bathrooms) OR :bathrooms IS NULL")
                    ->andWhere("(pm.key = 'Precio $' AND ABS(pm.value) >= :minPriceArs) OR :minPriceArs IS NULL")
                    ->andWhere("(pm.key = 'Precio $' AND ABS(pm.value) <= :maxPriceArs) OR :maxPriceArs IS NULL")      
                    ->andWhere("(pm.key = 'Precio u\$s' AND ABS(pm.value) >= :minPriceUsd) OR :minPriceUsd IS NULL")              
                    ->andWhere("(pm.key = 'Precio u\$s' AND ABS(pm.value) <= :maxPriceUsd) OR :maxPriceUsd IS NULL")
                    ->setParameter('city', $filter->getCity())
                    ->setParameter('operationType', $filter->getOperationType())
                    ->setParameter('propertyType', $filter->getPropertyType())
                    ->setParameter('bedrooms', $filter->getBedrooms())
                    ->setParameter('bathrooms', $filter->getBathrooms())
                    ->setParameter('minPriceArs', $filter->getMinPriceArs())
                    ->setParameter('maxPriceArs', $filter->getMaxPriceArs())
                    ->setParameter('minPriceUsd', $filter->getMinPriceUsd())                    
                    ->setParameter('maxPriceUsd', $filter->getMaxPriceUsd());
            
            $i = 0;
            foreach ($filter->getCharacteristics() as $characteristic) {
                $query = $query->andWhere("pm.key = 'Características' AND pm.value like :c" . $i)
                    ->setParameter('c'. $i, '%' . $characteristic . '%');
                $i++;
            }
            
            $i = 0;
            foreach ($filter->getAmenities() as $amenitie) {
                $query = $query->andWhere("pm.key = 'Amenities' AND pm.value like :a" . $i)
                    ->setParameter('a'. $i, '%' . $amenitie . '%');
                $i++;
            }
        }
        
        return $query->andWhere('p.published = 1')->getQuery()->getResult();
    }    
    
    public function findForEdit($id)
    {
        return $this->createQueryBuilder('p')
                    ->select('p, pi, pm, pmo')
                    ->leftJoin('p.images', 'pi')
                    ->leftJoin('p.meta', 'pm')
                    ->leftJoin('pm.option', 'pmo')
                    ->andWhere('p.id = :id')
                    ->setParameter('id', $id)
                    ->orderBy('pi.order, pmo.order', 'ASC')
                    ->getQuery()
                    ->getOneOrNullResult();
    }
    
    public function findForView($id)
    {
        return $this->createQueryBuilder('p')
                    ->select('p, pi, pm, pmo, t')
                    ->leftJoin('p.images', 'pi')
                    ->leftJoin('p.meta', 'pm')
                    ->leftJoin('pm.option', 'pmo')
                    ->leftJoin('p.tags', 't')
                    ->andWhere('p.id = :id')
                    ->setParameter('id', $id)
                    ->orderBy('pi.order, pmo.order', 'ASC')
                    ->getQuery()
                    ->getOneOrNullResult();
    }
    
    public function findWithImages($id)
    {
        return $this->createQueryBuilder('p')
                    ->select('p, pi')
                    ->leftJoin('p.images', 'pi')
                    ->andWhere('p.id = :id')
                    ->setParameter('id', $id)
                    ->orderBy('pi.order', 'ASC')
                    ->getQuery()
                    ->getOneOrNullResult();
    }
    
    public function getPublishedCount() {
        return $this->createQueryBuilder('p')
                    ->select('count(1)')
                    ->andWhere('p.published = 1')
                    ->getQuery()
                    ->getSingleScalarResult();
    }
    
}